@@ -8,47 +8,177 @@ from algebraic_hbm.ode import ODE_2nd_Order_Poly_Coeffs
 from algebraic_hbm.fourier_series import Fourier_Series
 from algebraic_hbm.utils import solve_subset_sum_problem, multinomial_coefficient, trigonometric_product_to_Fourier_series
 
-#Todo: Add doc-string.
 @dataclass
 class Algebraic_HBM():
+	"""
+	This class provides an implementation of the algebraic harmonic balance method (algebraic HBM) 
+	for second order ordinary differential equations with polynomial coefficients as proposed by [1].
 
-	# User-input
+	-------------------------------------------------
+	Theoretic background
+	-------------------------------------------------
+
+	Given an ODE of type
+
+		r(x) = mass*x'' + damping*x' + stiffness*x + f(x) - excitation
+	
+	with
+
+		f(x) = sum_{i=1}^q monomials[i]*x^i
+
+	where 'x' denotes a real-valued state function, 'f' a polynomial in 'x', 'excitation' a harmonic 
+	input function (e.g. '5*cos(a*t)') subject to an excitation frequency 'a'; 'mass', 'damping' 
+	and 'stiffness' are real coefficients, and 'monomomials' is a dictonary with (key,value) pairs
+	denoting the tuple (degree,coefficie)=('i', 'monomial[i]').
+
+	Given an ansatz order 'n', the method approximates 'x' by a truncated Fourier series 'x_n' and it
+	prescribes '2n+1' conditions '<r(x_n),phi_i>=0' (*) where '<,>' is the inner product on	the real 
+	Fourier space on which 'x_n' lives and 'phi_i' is the 'i'-th basis function for all 'i=0,1,...,2n'.
+	Solving (*) allows for determining the (real) Fourier coefficients 'c=(c[0],c[1],...,c[2n])'. This 
+	is the approach of the classical HBM. We instead derive a system of multivariate polynomials
+
+		R_i(c;a) = 0    for all    i=0,1,...,2n
+
+	where the 'R_i' are obtained by algebraically expanding the truncated Fourier series of the 
+	residual function 'r(x_n)', that is
+
+		r(t,x_n) = sum_{i=0}^{2*q*n} R_i(c;a) phi_i(t)
+
+	-------------------------------------------------
+	Usage
+	-------------------------------------------------
+
+	Given an ODE as defined through an instance 'ode' of 'ODE_2nd_Order_Poly_Coeffs' and an ansatz 
+	order 'n', an instance of 'Algebraic_HBM' is generated as
+
+		HBM = Algebraic_HBM(ODE=ode, order=n)
+
+	'Algebraic_HBM' can be used to
+
+	- generate the multivariate polynomials 'R_i' which map '(c,a)' onto 'R_i(c,a)' by invoking
+
+		HBM.generate_multivariate_polynomials()
+
+	- compile all multivariate polynomials 'R_i' into an algebraic system of equations 'F' and its 
+	  Jacobian 'DF=(DF[0],DF[1])' by calling
+	
+	    F, DF = HBM.compile()
+
+	- compute the coefficient matrix 'A' for the Macaulay framework by running
+
+		A = HBM.get_monomial_coefficient_matrix(a)
+
+	for some excitation frequency 'a'.
+
+	-------------------------------------------------
+	Examples
+	-------------------------------------------------
+
+	After installation via 'pip install algebraic-hbm' run the following examples.
+
+	Example 1: Computing a point on the frequency response curve
+
+		from algebraic_hbm import ODE_2nd_Order_Poly_Coeffs, Algebraic_HBM
+
+		softening_Duffing = ODE_2nd_Order_Poly_Coeffs.load_example(example="softening_Duffing")
+		n = 1
+		HBM = Algebraic_HBM(ODE=softening_Duffing, order=n)
+		HBM.generate_multivariate_polynomials()
+		F, DF = HBM.compile()
+
+		import numpy as np
+		from scipy.optimize import fsolve
+		c0, a = np.zeros(HBM.subspace_dim), 1.
+		c = fsolve(func=lambda x: F(x,a), fprime=lambda x: DF[0](x,a), x0=c0)
+		print(f"{c=}, norm(c)={np.linalg.norm(c):1.4f}")
+
+	Example 2: Computing coefficient matrix for Macaulay framework
+
+		from algebraic_hbm import softening_Duffing, Algebraic_HBM
+
+		n, a = 1, 1
+		HBM = Algebraic_HBM(ODE=softening_Duffing, order=n)
+		HBM.generate_multivariate_polynomials()
+		A = HBM.get_monomial_coefficient_matrix(a)
+
+	-------------------------------------------------
+	Documentation
+	-------------------------------------------------
+
+	The maintained repository including documentation is [2].
+
+	-------------------------------------------------
+	References
+	-------------------------------------------------
+
+	[1] Hannes Dänschel and Lukas Lentz. "An Algebraic Representation of the Harmonic Balance Method
+	    for Ordinary Differential Equations with Polynomial Coefficients"
+	[2] https://git.tu-berlin.de/hannes.daenschel/algebraic-hbm
+
+	"""
+
+	# User-input.
 	ODE:   ODE_2nd_Order_Poly_Coeffs = None
 	order:                       int = None
-	subspace_dim:                int = None
 	project_onto_subspace:      bool = True
 	include_constant_harmonic:  bool = True
 	use_jit:                    bool = True
 
-	_mass:      float = None
-	_damping:   float = None
-	_stiffness: float = None
-	_excitation: Fourier_Series = None
+	# Do not modify. These get populated from the ODE instance in '__post_init__'.
+	subspace_dim:                   int = None
+	_mass:                        float = None
+	_damping:                     float = None
+	_stiffness:                   float = None
+	_excitation:         Fourier_Series = None
 	_ODE_polynomial_coefficients: tuple = None
 
+	# Do not modify. These get populated in 'generate_multivariate_polynomials'.
 	_monomials_linear_part:    list = None
 	_monomials_nonlinear_part: list = None
 
-	#Todo: Add doc-string.
 	def __post_init__(self):
+		"""
+		Set some default values and compute the subspace dimension on which to project.
+		"""
 
+		# Set default values.
 		self._mass = self.ODE.mass if self.ODE.mass is not None else 0.
 		self._damping = self.ODE.damping if self.ODE.damping is not None else 0.
 		self._stiffness = self.ODE.stiffness if self.ODE.stiffness is not None else 0.
 		self._ODE_polynomial_coefficients = self.ODE.monomials if self.ODE.monomials is not None else {}
 
 		if not self.include_constant_harmonic:
+			# If we want to exclude the constant harmonic, message it and reduce the subspace dimension by one.
 			exclude_constant_harmonic = ", excluding constant harmonic"
-			self.subspace_dim = 2*self.order + 1
+			self.subspace_dim = 2*self.order
 		else:
 			exclude_constant_harmonic = ""
 			self.subspace_dim = 2*self.order + 1
+
 		print(f"Initialized HBM of order n={self.order}" + exclude_constant_harmonic)
 
 	def generate_multivariate_polynomials(self, project_onto_subspace: bool=None):
 		"""
-		Generate multivariate polynomials of linear and nonlinear part of HBM residual.
-		Also optionally removes constant harmonic from system.
+		Generate multivariate polynomials 'R_i' of the linear and nonlinear part of the
+		HBM residual 'r(x_n)'. Using the multiindex notation, the 'R_i' are represented
+		internally by a list of monomials such that
+
+			R_i(c;a) = b_{w_{1,i}} c^{w_{1,i}} + ... + b_{w_{N,i}} c^{w_{N_i,i}}
+
+		of length 'sum_i N_i'. Each of the monomials itself is a tuple 
+
+			(i, b_{w_{j,i}}, w_{j,i})    for all    j=1,...,N_i , i=0,1,...,2n
+		  
+		Note that internally 'c' is treated as an extension of the HBM coefficient
+		vector that is extend by the excitation frequency 'a', that is we are using
+
+			c = (c_0,c_1,...,c_{2n},a)
+
+		until either '.compile()' or '.get_monomial_coefficient_matrix(a)' is called.
+
+		Also optionally removes constant harmonic from system,
+		that is we are then not considering 'R_0' and the HBM coefficient vector 
+		'c = (c_0,c_1,...,c_{2n})' is changed to 'c = (c_1,...,c_{2n})'.
 		"""
 		
 		# Generate multivariate polynomials of linear part of HBM residual.
@@ -60,8 +190,11 @@ class Algebraic_HBM():
 		# Remove constant harmonic from system.
 		self.remove_constant_harmonic()
 
-	#Todo: Add doc-string.
 	def generate_linear_part(self):
+		"""
+		Generate linear part of multivariate polynomials 'R_i'.
+		For more information read doc-string of '.generate_multivariate_polynomials()'.
+		"""
 
 		print(f"Generating linear part of algebraic Fourier coefficients of HBM residual ...", end="")
 
@@ -93,8 +226,11 @@ class Algebraic_HBM():
 
 		print(" done")
 
-	#Todo: Add doc-string.
 	def generate_nonlinear_part(self):
+		"""
+		Generate nonlinear part of multivariate polynomials 'R_i'.
+		For more information read doc-string of '.generate_multivariate_polynomials()'.
+		"""
 		
 		print(f"Generating nonlinear part of algebraic Fourier coefficients of HBM residual ...", end="")
 
@@ -168,8 +304,6 @@ class Algebraic_HBM():
 			self._monomials_nonlinear_part = [(l, monomial_coefficient, multidegree) for l, monomial_coefficient, multidegree in self._monomials_nonlinear_part if l <= 2*self.order]
 			print(" done")
 		
-		
-						
 	def remove_constant_harmonic(self):
 		"""
 		For certain ODEs with certain periodic forcing the harmonic with index 0 will be constantly zero
@@ -179,11 +313,13 @@ class Algebraic_HBM():
 			self._monomials_linear_part = [(l-1, monomial_coefficient, multidegree[1:]) for l, monomial_coefficient, multidegree in self._monomials_linear_part if l > 0 and multidegree[0] == 0]
 			self._monomials_nonlinear_part = [(l-1, monomial_coefficient, multidegree[1:]) for l, monomial_coefficient, multidegree in self._monomials_nonlinear_part if l > 0 and multidegree[0] == 0]
 	
-	#Todo: Add doc-string.
 	@staticmethod
-	def generate_product_indices_and_multinomial_coefficients(m: int, p: int, K_p: list) -> list:
+	def generate_index_tuples_and_multinomial_coefficients(m: int, p: int, K_p: list) -> list:
+		"""
+		Returns index tuples and associated multinomial coefficients.
+		"""
 		
-		index_tuples, multinom = [], []
+		index_tuples, multinoms = [], []
 		for K in K_p:
 
 			# Get all integer tuples of length 'len(K)' that are combinations/permutations of
@@ -199,9 +335,9 @@ class Algebraic_HBM():
 				# tuples where for each 'j=1,...,N' the index 'i_j' occurs 'k_j' times.
 				index_tuples.append(tuple(t for j, k in zip(I,K) for t in (j,)*k))
 
-			multinom += (multinomial_coefficient(p,K),)*len(tuple(J))
+			multinoms += (multinomial_coefficient(p,K),)*len(tuple(J))
 
-		return index_tuples, multinom
+		return index_tuples, multinoms
 	
 	#Todo: Add non-jitted branch
 	def compile(self):
