@@ -4,14 +4,16 @@ from __future__ import annotations
 import sys
 
 # Data types.
-from dataclasses import dataclass
+from dataclasses import dataclass, field
+from collections.abc import Sequence
 from numbers import Number
 from fractions import Fraction
-from collections.abc import Sequence
 
 # Math.
 import numpy as np
 
+DEFAULT_COEFFICIENTS = (0.,)
+
 @dataclass
 class Fourier_Series():
 	'''
@@ -33,8 +35,8 @@ class Fourier_Series():
 		- get_rms: compute the root-mean-square of 'f'
 	'''
 
-	coefficients: np.ndarray | list | tuple = np.array([0])
-	frequency:                        float = None
+	coefficients: Sequence = field(default_factory=lambda: DEFAULT_COEFFICIENTS)
+	frequency:       float = None
 
 	# Parameters for pretty printing.
 	variable_name:              str = 't'
@@ -45,11 +47,13 @@ class Fourier_Series():
 
 	def __post_init__(self):
 
-		# Make sure that the coefficients are always of np.ndarray type.
-		if type(self.coefficients) in [list, tuple]:
+		# Make sure coefficients is a valid type.
+		if not (isinstance(self.coefficients, Sequence) or isinstance(self.coefficients, np.ndarray)):
+			raise TypeError(f"Expecting {Sequence.__name__} or {np.__name__ + '.' + np.ndarray.__name__}.")
+
+		# Make sure coefficients is of type numpy.ndarray.
+		if not isinstance(self.coefficients, np.ndarray):
 			self.coefficients = np.array(self.coefficients)
-		if type(self.coefficients) is not np.ndarray:
-			raise TypeError("'coefficients' must must either be of type tuple, list or np.ndarray")
 
 		# We always want to keep the sine-cofficient, even if it is zero.
 		if not self.coefficients.shape[0] % 2:
@@ -62,6 +66,10 @@ class Fourier_Series():
 		self.coefficients[int(item)] = value
 
 	def __eq__(self, other):
+		"""
+		We want to be able to compare instances of this class. For the comparison
+		we only consider the 'coefficients'- and 'frequency'-attribute
+		"""
 
 		# We only want to compare the coefficients and the frequency.
 		coeffs = self.coefficients == other.coefficients
@@ -72,14 +80,27 @@ class Fourier_Series():
 		return coeffs and freqs
 
 	def __call__(self, T: Number | np.ndarray) -> Number | np.ndarray | tuple:
+		"""
+		Method that evaluates the function that is represented by an instance of this class."
+		"""
 		return self.eval(T)
 
 	def get_order(self) -> int:
-		return int(self.coefficients.shape[0]/2)
+		"""
+		Return the order of the Fourier series, that is return the number 'n' as in the class doc-string.
+		"""
+		n = int(self.coefficients.shape[0]/2)
+		return n
+	
+	def get_dimension(self) -> int:
+		"""
+		Return the dimension of the real Fourier space this Fourier series lives in. That is, 
+		returns the number '2n+1' where 'n' as in the class doc-string.
+		"""
+		return int(self.coefficients.shape[0])
 	
 	def get_fundamental_period(self) -> float:
 		'''Returns the largest period of the Fourier series.'''
-
 		return 2*np.pi/self.frequency/self.get_order()
 
 	def derivative(self) -> Fourier_Series:
@@ -245,7 +266,7 @@ class Fourier_Series():
 
 		return np.sqrt(c[0]**2 + c[1:].dot(c[1:])/2)
 
-	''' The methods are for pretty printing '''
+	''' pretty printing '''
 
 	def __str__(self) -> str:
 		return self._pretty_repr()
@@ -254,20 +275,29 @@ class Fourier_Series():
 		print(self._pretty_repr())
 	
 	def _pretty_repr(self) -> str:
+		"""
+		repr-method that returns a formated string that represents this Fourier series as a
+		sum of weighted trigonometric basis functions.
+		"""
 
 		if all(self.coefficients == 0):
 			return str(0)
 
 		s = ''
 		k = 0
+
+		# Iterate over each coefficient of the series.
 		for c in self:
 
+			# We can ignore all terms that have a zero-coefficient.
 			if c == 0:
 				k += 1
 				continue
-
+			
+			# Take the absolute value since we print each coefficient 'c' manually.
 			val = np.abs(c)
 
+			# Properly format the leading sign symbol.
 			if c > 0 and s != '':
 				s += " + "
 			elif c < 0 and s != '':
@@ -276,10 +306,14 @@ class Fourier_Series():
 				s += "-"
 
 			
+			# The first coefficient is always the once that has the constant basis function.
+			# So we just print the coefficient and no basis function.
 			if k == 0:
 				s += self.pretty_constant(val)
+				print(val)
 
 			else:
+				# Now we want to determin the index of and the basis function that we want to print.
 
 				if k%2: # odd -> c*cos(k*x)
 					j, operator = int((k-1)/2) + 1, "cos"
@@ -287,12 +321,15 @@ class Fourier_Series():
 				else: # even -> c*sin(k*x)
 					j, operator = int(k/2), "sin"
 				
+				# Concatenate everything. The pattern looks something like 'coefficient*cos(j*frequency*time)'.
 				s += self.pretty_factor(val) + operator + "(" + self.pretty_argument(j,self.frequency) + ")"
 
+			# Increments the index of the current coefficient.
 			k += 1
 
 		return s
 
+	#Todo: Check if this method is still required.
 	def get_angles(self):
 		
 		# Get cosine and sine coefficient indices.
@@ -308,13 +345,15 @@ class Fourier_Series():
 		if np.any(sin):
 			frequencies[sin] = (sin+1)/2
 
-		# raise NotImplementedError
 		return self.frequency * frequencies
 
-	def fractionate(self, a):
+	def fractionate(self, z):
+		"""
+		Approximate number 'z' by a rational number and return information on it.
+		"""
 
 		# Get fraction representation.
-		nom, denom = Fraction(a).as_integer_ratio()
+		nom, denom = Fraction(z).as_integer_ratio()
 
 		is_integer = False
 		is_rational = False
@@ -327,51 +366,52 @@ class Fourier_Series():
 
 		return nom, denom, is_integer, is_rational
 
-	#TODO: finish implementation: what if nom and denom are huge -> ugly -> make pretty
-	def pretty_constant(self, a):
-		x = self.variable_name
+	#Todo: improve implementation: what if nom and denom are huge -> ugly -> make pretty
+	def pretty_constant(self, z):
+		"""
+		Formats number 'a' in a pretty way.
+		"""
 
-		nom, denom, is_integer, is_rational = self.fractionate(a)
+		# Approximate 'z' as a rational number.
+		nom, _, is_integer, is_rational = self.fractionate(z)
 
+		# Straight forward formating if 'z' is integer.
 		if is_integer:
-			
-			if nom == 1:
-				return "1"
-			if nom == -1:
-				return "-1"
-			else:
-				return str(nom)
-
+			return str(nom)
 		else:
-
 			if is_rational:
-				return self.pretty_rational(a)
+				# Pretty format the ration number.
+				return self.pretty_rational(z)
 			else:
+				#Todo: Is this case relevant, can it occur? Deprecated?
 				raise NotImplementedError()
 
-	#TODO: finish implementation: what if nom and denom are huge -> ugly -> make pretty
+	#Todo: improve implementation: what if nom and denom are huge -> ugly -> make pretty
 	def pretty_argument(self, j, f):
+		"""
+		Pretty format the argument 'a' of either cos(a) or sin(a).
+		"""
 
-		harmonic  = ""
+		harmonic_index  = ""
 		frequency = self.frequency_name
 		variable  = self.variable_name
 		mul_symbol_1 = self.multiplication_symbol
 		mul_symbol_2 = self.multiplication_symbol
 
-		if not self.print_dot_in_argument:
-			mul_symbol = ""
-
 		# Prepare harmonic string.
 		if j == 1:
-			harmonic = ""
+			# We dont print the one.
+			harmonic_index = ""
 		elif j == -1:
-			harmonic = "-"
+			harmonic_index = "-"
 		else:
-			harmonic = str(j)
+			harmonic_index = str(j)
 
-		# Prepare frequency string.
+		# Format frequency.
 		if f is not None:
-			nom, denom, is_integer, is_rational = self.fractionate(f)
+			# f is given as a number.
+
+			nom, _, is_integer, is_rational = self.fractionate(f)
 
 			if is_integer:
 				
@@ -388,21 +428,28 @@ class Fourier_Series():
 					frequency = self.pretty_rational(f)
 				else:
 					raise NotImplementedError()
+		else:
+			# f is not given so we treat it as being one so we dont print it.
+			frequency = ""
 
-		if harmonic == "":
+		if harmonic_index == "":
 			mul_symbol_1 = ""
 		if frequency == "":
 			mul_symbol_2 = ""
 
-		return harmonic + mul_symbol_1 + frequency + mul_symbol_2 + variable
+		return harmonic_index + mul_symbol_1 + frequency + mul_symbol_2 + variable
 
 	#TODO: finish implementation: what if nom and denom are huge -> ugly -> make pretty
-	def pretty_factor(self, a):
+	def pretty_factor(self, z):
+		"""
+		Return formated coefficient/weight 'a' that is multiplied by the basis functions.
+		"""
 
-		nom, denom, is_integer, is_rational = self.fractionate(a)
+		# Approximate 'z' by rational number.
+		nom, denom, is_integer, is_rational = self.fractionate(z)
 
 		if is_integer:
-			
+			# Treat the integer case differently.			
 			if nom == 1:
 				return ""
 			if nom == -1:
@@ -413,9 +460,11 @@ class Fourier_Series():
 		else:
 
 			if is_rational and np.log10(nom) <= 4 and np.log10(denom) <= 4:
-				fac = self.pretty_rational(a)
+				# If the nominator is not too large we print 'z' as a rational number.
+				fac = self.pretty_rational(z)
 			else:
-				fac = f"{a:1.{self.print_coefficient_decimals}f}"
+				# Print it as a floating point number with given number of decimals.
+				fac = f"{z:1.{self.print_coefficient_decimals}f}"
 
 				# Now remove trailing zeros after the period.
 				while True:
@@ -427,12 +476,17 @@ class Fourier_Series():
 			return fac + self.multiplication_symbol
 
 	def pretty_rational(self, z):
+		"""
+		Pretty format the rational number 'z'.
+		"""
 
 		nom, denom = Fraction(z).as_integer_ratio()
-		
-		if nom*1e-3 > 1: # np.abs(z) < 1 and 
+		tol = 1e3
+		if nom > tol:
+			# Just format as this if the nominator is too large.
 			s = f"{z:1.2f}"
 		else:
+			# Print a rational number.
 			s = str(nom) + "/" + str(denom) 
 
 		return s
\ No newline at end of file
